<?php
/*
 * Shibboleth Log viewer. Operations.
 *
 *  Jorge López Pérez <jorgelp@us.es>
 */


/*
 * Returns interface for log 'name'
 */

function loginterface($name, $lines, $type) {
?>
   <h2><?php echo $name ?>.log</h2>
   <div id="<?php echo $name ?>" class="log">
    <div id="<?php echo $name?>-content">
<?php 
  	echo readlog($name.".log", $lines, $type);
?>
   </div>
   <a name="bottom<?php echo $name ?>" id="bottom<?php echo $name ?>"></a>
   </div>
   <form action="<?php echo $_SERVER['SCRIPT_NAME'] ?>" method="get">
    <label for="<?php echo $name ?>lines">Lines to show:</label>
    <input type="text" name="<?php echo $name ?>lines" id="<?php echo $name ?>lines" 
	maxlength="4" size="4" value="<?php echo $lines; ?>" />
    <input type="submit" name="<?php echo $name ?>linessubmit" id="<?php echo $name ?>linessubmit"
	class="changelinesbutton" value="Change" />
   <span><a id="refresh_<?php echo $name ?>_<?php echo $type ?>" class="refresh" href="#bottom<?php echo $name ?>">refresh</a></span>
   </form>


<?php
}



/*
 * Returns last $lines lines from file logs_dir / $file
 */

function readlog($file, $lines = 30, $type = 'sp') {
    global $paths;

	if ($type == 'sp') {
	  	$logs_dir = $paths['sp_logs'];
	} else {
	  	$logs_dir = $paths['idp_logs'];
	}

    // Security check
    $file = basename($file);

    $fullpath = $logs_dir . '/' . $file;

    $command = sprintf("%s -%d %s 2>&1", 
        $paths['tail'], $lines, $fullpath);

    $fp = popen($command, 'r');

    if (!$fp) {
        return 'Error creating pipe';
    } else {
        $result = '';
        while (!feof($fp)) {
            $line = fgets($fp, 4096);

			if (!dismiss($line, $type)) {

				// Postprocessing
				$line = highlight($line, $type);
				$result .= preg_replace('/$/', '<br />', $line)."\n";
			}
        }

        pclose($fp);
    }

    return $result;
}

/*
 * Syntax highlighting
 */

function highlight($text, $type = 'sp') {
  if ($type == 'sp') {

	$pattern = '/^(\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2}) (\w+) ([\w.-]+) (\[\d+\]|:) (.+)$/';

	$match = preg_match($pattern, $text, $pieces);
	if ($match == 0 || $match == FALSE) {
	  return "no highlight: ".$text;
	} else {
	  $line = '<span class="logtimestamp">'.$pieces[1].'</span> ';
	  $line .= '<span class="log'.strtolower($pieces[2]).'">'.$pieces[2].'</span> ';
	  $line .= '<span class="logsource">'.$pieces[3].'</span> ';
	  $line .= '<span class="logadditional">'.$pieces[4].'</span> ';

	  // More changes
	  $rest = $pieces[5];
	  $subpatterns = array(
		  // IDs
		  '/(\((ID(=|:\s))?_([\w\d]+)\))/',
		  // URNs
		  '/(urn:[\w0-9\.:-]+)/',
		  // IPs
		  '/(\d+\.\d+\.\d+\.\d+)/',
		  // URLs
		  '/(http[s]?:\/\/[\w0-9\.\/-]+(:\d+)?[\w0-9\.\/-]+)/'
		  );
	  $changes = array(
		  '<span class="logid">\1</span>',
		  '<span class="logurn">\1</span>',
		  '<span class="logip">\1</span>',
		  '<span class="logurl">\1</span>',
		  );
	  $rest = preg_replace($subpatterns, $changes, $rest);

	  $line .= '<span class="logrest">'.$rest.'</span> ';
	}

  } else {
	// IdP highlight
 	$patterns = array(
		// Timestamp
		'/^(\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2})/',
		// IDs
		'/(_[\w0-9]{32})/',
		// Principal name
		'/(of principal|user|principal[:]?) \(([\w0-9\.-]+)\)/',
		// Provider name
		'/(provider) \(([\w0-9\.:-]+)\)/',
		// URNs
		  '/(urn:[\w0-9\.:-]+)/',
		// Trick for principal name
		'/(Found) (\w+) (attribute\(s\)) for ([\w0-9\.-]+)/',
		// XML block
		'/(&lt;.+&gt;)$/',
		// URLs
		'/(http[s]?:\/\/[\w0-9\.\/-]+(:\d+)?[\w0-9\.\/-]+)/'
	  );
	$changes = array(
		'<span class="logtimestamp">\1</span>',
		'<span class="logid">\1</span>',
		'\1 (<span class="logprincipal">\2</span>)',
		'\1 (<span class="logprovider">\2</span>)',
		'<span class="logurn">\1</span>',
		'\1 <span class="lognum">\2</span> \3 (<span '
		  .'class="logprincipal">\4</span>)',
		'<span class="logxml">\1</span>',
		'<span class="logurl">\1</span>'
	);
	
	$line = htmlentities($text);
	$line = preg_replace($patterns, $changes, $line);


  }
  return $line;
}


/*
 * Dismiss line
 */

function dismiss($line, $type = 'sp') {
	if ($line == '')
		return true;
	else
		return false;
}

/*
 * Footer
 */

function footer() {
  echo '<div id="footer">shiblogs 0.1.2</div>';
}

// vim:sw=2
?>
